--------------------------------------------------------------------------------
-- Animation
--------------------------------------------------------------------------------
Animation = {}

function Animation:new()
	local o = {}
	setmetatable(o, self)
	self.__index = self
	
	Animation:construct(o)
	
	return o
end

function Animation:construct(o)
	o.frames = {}
	o.time = 0.0
	o.framerate = 10
	o.frameno = 1
	o.numframes = 0
	o.sizex = 64
	o.sizey = 64
	o.loop = true
	o.finished = false
	o.scale = 1.0
	
	o.fade = false
	o.fadetime = 0.5
	o.fadealpha = 0.0
	
	o.nextframe = 1
end

function Animation:update(dt)
	
	local r = 1.0/self.framerate
	
	if not self.finished then
		self.time = self.time + dt
		
		self.fadealpha = math.min( math.max(self.time, 0.0) / self.fadetime, 1.0)
		
		if self.time > r then
		
			self.time = 0.0
		
			self.frameno = self.frameno + 1
			
			self.nextframe = self.frameno + 1
			if self.nextframe > self.numframes then
				self.nextframe = 1
				if not self.loop then
					self.nextframe = self.numframes
				end
			end
			
			
			if self.frameno > self.numframes then
				
				if self.loop then
					self:play()
				else 
					self:stop()
				end
			end
		end	
	end

	
	
end

function Animation:draw(x,y, dir, zoom_)
	
	if not zoom_ then zoom_ = 1.0 end
	
	if self.numframes <= 0 then
		return
	end
	
	if self.fade then
		-- save current color state
		local r,g,b,a = love.graphics.getColor()

		-- Which frames should we blend between?
		local thisframe = self.frameno
		local nextframe = self.nextframe
		if nextframe > self.numframes then
			nextframe = self.numframes
			
			if self.loop then
				nextframe = 1
			end
		end
		
		local alpha = self.fadealpha
		if alpha == 1.0 then
			alpha = 0.0
		end
		
		self:drawFrame(x, y, dir, zoom_, thisframe)
		
		-- Fade this frame out
		love.graphics.setColor(r,g,b, a*(1.0-alpha))
		self:drawFrame(x, y, dir, zoom_, thisframe)
		
		-- At the same time, fade the next one in
		love.graphics.setColor(r,g,b, a*alpha)
		self:drawFrame(x, y, dir, zoom_, nextframe)
		
		-- reset color state
		love.graphics.setColor(r,g,b,a)
	
	else
		self:drawFrame(x,y,dir,zoom_,self.frameno)
	end
	
end

function Animation:drawFrame(x, y, dir, zoom_, frameno)
	love.graphics.draw( self.frames[frameno], x, y, 0, dir * self.scale * zoom_, 1 * self.scale * zoom_, (self.sizex / 2), self.sizey)
end

function Animation:addFrame(f)
	table.insert(self.frames, f)
	self.numframes = self.numframes + 1
end

function Animation:play()
	self.frameno = 1
	self.finished = false
	self.time = 0.0
end

function Animation:stop()
	self.frameno = self.numframes
	self.finished = true
end

function Animation:isFinished()
	return o.finished
end

function Animation:setLooping(looping)
	self.loop = looping
end






function loadAnimation(path, sizex, sizey, n, framerate, scale, loop)

	local animation = Animation:new()

	for i = 1, n do
		local img = loadImage(path .. i .. ".png")
		animation:addFrame(img)
	end
	animation.loop = loop
	animation.scale = scale
	animation.framerate = framerate
	animation.sizex = sizex
	animation.sizey = sizey
	
	return animation
end

